#ifndef PCL_WRAPPER_FILTER_HPP
#define PCL_WRAPPER_FILTER_HPP
// filter.hpp
#pragma once
#include <pcl/point_cloud.h>
#include "point_types.hpp"

namespace pcl_wrapper {

    template<typename PointT>
    class FilterBase {
    public:
        virtual ~FilterBase() = default;
        virtual void setInputCloud(typename pcl::PointCloud<PointT>::Ptr cloud) = 0;
        virtual typename pcl::PointCloud<PointT>::Ptr getFilteredCloud() = 0;
    };

    // filter.hpp 中追加声明（不要写实现体）
    template<typename PointT>
    class VoxelGridFilterImpl : public FilterBase<PointT> {
    public:
        void setLeafSize(float ls);
        void setInputCloud(typename pcl::PointCloud<PointT>::Ptr cloud) override;
        typename pcl::PointCloud<PointT>::Ptr getFilteredCloud() override;

    private:
        typename pcl::PointCloud<PointT>::Ptr inputCloud;
        float leafSize = 0.1f;
    };

} // namespace pcl_wrapper

#endif //PCL_WRAPPER_FILTER_HPP
