
// pcl_wrapper/src/io.cpp
#include "io.hpp"

namespace pcl_wrapper {

    template<typename PointT>
    typename pcl::PointCloud<PointT>::Ptr loadPCD(const std::string& filename) {
        typename pcl::PointCloud<PointT>::Ptr cloud(new pcl::PointCloud<PointT>());
        if (pcl::io::loadPCDFile<PointT>(filename, *cloud) == -1) {
            throw std::runtime_error("Failed to load PCD file: " + filename);
        }
        return cloud;
    }

    template<typename PointT>
    void savePCD(const std::string& filename, const typename pcl::PointCloud<PointT>::Ptr& cloud) {
        if (pcl::io::savePCDFileBinary(filename, *cloud) == -1) {
            throw std::runtime_error("Failed to save PCD file: " + filename);
        }
    }

// 显式实例化
    template pcl::PointCloud<pcl::PointXYZ>::Ptr loadPCD<pcl::PointXYZ>(const std::string&);
    template pcl::PointCloud<pcl::PointXYZI>::Ptr loadPCD<pcl::PointXYZI>(const std::string&);
    template void savePCD<pcl::PointXYZ>(const std::string&, const pcl::PointCloud<pcl::PointXYZ>::Ptr&);
    template void savePCD<pcl::PointXYZI>(const std::string&, const pcl::PointCloud<pcl::PointXYZI>::Ptr&);

} // namespace pcl_wrapper
